use serde::{Deserialize, Serialize};
use std::collections::HashMap;
use std::path::PathBuf;
use std::sync::Arc;
use tokio::io::AsyncWriteExt;
use tokio::sync::RwLock;

#[derive(Clone)]
pub struct State {
    pub config: Arc<Config>,
    pub inner: Arc<RwLock<StateInner>>,
    pub matrix_client: Arc<matrix_sdk::Client>,
}

#[derive(Clone, Deserialize)]
pub struct Config {
    pub bind_address: String,
    pub matrix_id: String,
    pub matrix_room: String,
    pub matrix_store: PathBuf,
    pub matrix_password: String,
    pub monzo_persist: PathBuf,
    pub monzo_client_id: String,
    pub monzo_client_secret: String,
    pub base_uri: String,
}

#[derive(Default)]
pub struct StateInner {
    pub monzo_state: Option<MonzoState>,
}

#[derive(Clone, Serialize, Deserialize)]
pub struct MonzoState {
    pub access_token: String,
    pub access_token_expiry: u64,
    pub refresh_token: String,
    /// Account IDs to names
    pub accounts: HashMap<String, String>,
}

pub async fn persist_monzo_state(state: &State) -> anyhow::Result<()> {
    let monzo_state = state
        .inner
        .read()
        .await
        .monzo_state
        .clone()
        .ok_or_else(|| anyhow::anyhow!("No monzo state to persist"))?;
    let mut file = tokio::fs::File::create(&state.config.monzo_persist).await?;
    let json_data = serde_json::to_vec(&monzo_state)?;
    file.write_all(&json_data).await?;
    Ok(())
}
